package cz.muni.fi.pb162.test.concentration;


public class ConcentrationImpl implements ConcentrationDesk {

    private Card[] cards;
    private int firstCardIndex;
    
    /**
	* @param cards Array of cards for the game.
	* @throws IllegalCardSetException If given cards are not valid 
	*         playing cards for the concentration game, 
	*         i.e. the array is null, some card in the array is null 
	*         or cards do not make pairs.
	*/
   public ConcentrationImpl(Card[] cards) throws IllegalCardSetException {
        if(cards == null) {
            throw new IllegalCardSetException("Array of cards is null!");
        }
        for(int i = 0; i < cards.length; i++) {
            if(cards[i] == null) {
                throw new IllegalCardSetException("Array of cards contains a card that is null!");
            }
        }
        // Sem este doplnit kontrolu, ci su dane karty poparovane (potom prejde aj cely test)
        this.cards = cards;
        firstCardIndex = -1;
   }
   
     /**
     * Uncovers first card.
     * 
     * @param index Index of the card to be uncovered, 0 = first card
     * @return Uncovered card.
     * @throws NoCardException If there is no card at the index.
     * @throws IllegalOperationException If this method has been already 
     *         invoked without subsequent invocation of uncoverSecondCard().
     */
    public Card uncoverFirstCard(int index) throws ConcentrationException {
        Card firstCard = cardAt(index);
        if(firstCard == null) {
            throw new NoCardException("There is no such a card!");
        }
        if(firstCardIndex >= 0) {
            throw new IllegalOperationException("Uncover a second card!");
        } 
        firstCardIndex = index;
        return firstCard;
    }
    
    /**
     * Uncovers second card. If both the uncovered cards on the desk 
     * match each other then they are removed from the desk.
     * If they don't match they are covered back.
     * 
     * @param index Index of the card to be uncovered, 0 = first card.
     * @throws NoCardException If there is no card at the index.
     * @throws IllegalOperationException If the first card was not uncovered yet, 
     *         i.e. the uncoverFirstCard() method was not invoked.
     * @throws UncoveredCardException If the second card is the first 
     *         (already uncovered) card
     */
    public boolean uncoverSecondCard(int index) throws ConcentrationException {
        Card secondCard = cardAt(index);
        if(secondCard == null) {
            throw new NoCardException("There is no such a card!");
        }
        if (firstCardIndex == -1) {
            throw new IllegalOperationException("Uncover a first card!");
        }
        if (firstCardIndex == index) {
            throw new UncoveredCardException("Why do you try to uncover the same card?!");
        }
        boolean isOK = false;
        if(secondCard.equals(cardAt(firstCardIndex))) {
            isOK = true;
            cards[index] = null;
            cards[firstCardIndex] = null;
        }
        firstCardIndex = -1;
        return isOK;
    }
    
    /**
     * @param index Index of the card to be returned, 0 = first card.
     * @return Card at the given index or null.
     */
    public Card cardAt(int index) {
        if(index < 0 || index >= cards.length) {
            return null;
        }
        return cards[index];
    }
}
